# This Python file uses the following encoding: utf-8
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


STEEM_100_PERCENT = 10000
STEEM_1_PERCENT = 100
STEEM_REVERSE_AUCTION_WINDOW_SECONDS = 1800
STEEM_VOTE_REGENERATION_SECONDS = 432000
STEEM_ROOT_POST_PARENT = ''
